""" Classes to convert entered content into HTML

		Copyright (c) 2003 Colin Stewart (http://www.owlfish.com/)
		All rights reserved.
		
		Redistribution and use in source and binary forms, with or without
		modification, are permitted provided that the following conditions
		are met:
		1. Redistributions of source code must retain the above copyright
		   notice, this list of conditions and the following disclaimer.
		2. Redistributions in binary form must reproduce the above copyright
		   notice, this list of conditions and the following disclaimer in the
		   documentation and/or other materials provided with the distribution.
		3. The name of the author may not be used to endorse or promote products
		   derived from this software without specific prior written permission.
		
		THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
		IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
		OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
		IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
		INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
		NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
		DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
		THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
		(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
		THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""

import os, sgmllib, StringIO, cgi, re

try:
	import logging
except:
	from simpletal import DummyLogger as logging
	
# All elements have a value of (type, tagOptional)
BLOCK_TYPE=0
INLINE_TYPE=1

TAG_OPTIONAL=1
TAG_REQUIRED=0
TAG_FORBIDDEN=2

# Taken from http://www.w3.org/TR/html4/index/elements.html
# Not complete, only ones I think I'll use in here.
HTML_TAGS={'A': (INLINE_TYPE, TAG_REQUIRED), 'ABBR': (INLINE_TYPE, TAG_REQUIRED), 'ACRONYM': (INLINE_TYPE, TAG_REQUIRED)
			,'ADDRESS': (BLOCK_TYPE, TAG_REQUIRED)
			,'APPLET': (BLOCK_TYPE, TAG_REQUIRED)
			,'AREA': (INLINE_TYPE, TAG_FORBIDDEN)
			,'B': (INLINE_TYPE, TAG_REQUIRED)
			,'BASE': (INLINE_TYPE, TAG_FORBIDDEN)
			,'BASEFONT': (INLINE_TYPE, TAG_FORBIDDEN)
			,'BDO': (INLINE_TYPE, TAG_REQUIRED)
			,'BIG': (INLINE_TYPE, TAG_REQUIRED)
			,'BLOCKQUOTE': (BLOCK_TYPE, TAG_REQUIRED)
			,'BR': (INLINE_TYPE, TAG_FORBIDDEN)
			,'BUTTON': (INLINE_TYPE, TAG_REQUIRED)
			,'CAPTION': (INLINE_TYPE, TAG_REQUIRED)
			,'CENTER': (INLINE_TYPE, TAG_REQUIRED)
			,'CITE': (INLINE_TYPE, TAG_REQUIRED)
			,'CODE': (INLINE_TYPE, TAG_REQUIRED)
			,'COL': (INLINE_TYPE, TAG_FORBIDDEN)
			,'COLGROUP': (INLINE_TYPE, TAG_OPTIONAL)
			,'DD': (INLINE_TYPE, TAG_OPTIONAL)
			,'DEL': (INLINE_TYPE, TAG_REQUIRED)
			,'DFN': (INLINE_TYPE, TAG_REQUIRED)
			,'DIR': (BLOCK_TYPE, TAG_REQUIRED)
			,'DIV': (BLOCK_TYPE, TAG_REQUIRED)
			,'DL': (BLOCK_TYPE, TAG_REQUIRED)
			,'DT': (INLINE_TYPE, TAG_OPTIONAL)
			,'EM': (INLINE_TYPE, TAG_REQUIRED)
			,'FIELDSET': (BLOCK_TYPE, TAG_REQUIRED)
			,'FONT': (INLINE_TYPE, TAG_REQUIRED)
			,'FORM': (BLOCK_TYPE, TAG_REQUIRED)
			,'H1': (BLOCK_TYPE, TAG_REQUIRED)
			,'H2': (BLOCK_TYPE, TAG_REQUIRED)
			,'H3': (BLOCK_TYPE, TAG_REQUIRED)
			,'H4': (BLOCK_TYPE, TAG_REQUIRED)
			,'H5': (BLOCK_TYPE, TAG_REQUIRED)
			,'H6': (BLOCK_TYPE, TAG_REQUIRED)
			,'HR': (BLOCK_TYPE, TAG_FORBIDDEN)
			,'I': (INLINE_TYPE, TAG_REQUIRED)
			,'IFRAME': (INLINE_TYPE, TAG_REQUIRED)
			,'IMG': (INLINE_TYPE, TAG_FORBIDDEN)
			,'INPUT': (INLINE_TYPE, TAG_FORBIDDEN)
			,'INS': (INLINE_TYPE, TAG_REQUIRED)
			,'KBD': (INLINE_TYPE, TAG_REQUIRED)
			,'LABEL': (INLINE_TYPE, TAG_REQUIRED)
			,'LEGEND': (INLINE_TYPE, TAG_REQUIRED)
			,'LI': (BLOCK_TYPE, TAG_OPTIONAL)
			,'LINK': (INLINE_TYPE, TAG_FORBIDDEN)
			,'MAP': (BLOCK_TYPE, TAG_REQUIRED)
			,'MENU': (BLOCK_TYPE, TAG_REQUIRED)
			,'NOFRAMES': (BLOCK_TYPE, TAG_REQUIRED)
			,'NOSCRIPT': (BLOCK_TYPE, TAG_REQUIRED)
			,'OBJECT': (INLINE_TYPE, TAG_REQUIRED)
			,'OL': (BLOCK_TYPE, TAG_REQUIRED)
			,'OPTGROUP': (INLINE_TYPE, TAG_REQUIRED)
			,'OPTION': (INLINE_TYPE, TAG_OPTIONAL)
			,'P': (BLOCK_TYPE, TAG_OPTIONAL)
			,'PARAM': (INLINE_TYPE, TAG_FORBIDDEN)
			,'PRE': (BLOCK_TYPE, TAG_REQUIRED)		# Flagged as block to avoid <br> tags being added.
			,'Q': (INLINE_TYPE, TAG_REQUIRED)
			,'S': (INLINE_TYPE, TAG_REQUIRED)
			,'SAMP': (INLINE_TYPE, TAG_REQUIRED)
			,'SCRIPT': (BLOCK_TYPE, TAG_REQUIRED)		# Flagged as block to avoid <br> tags being added.
			,'SELECT': (INLINE_TYPE, TAG_REQUIRED)
			,'SMALL': (INLINE_TYPE, TAG_REQUIRED)
			,'SPAN': (INLINE_TYPE, TAG_REQUIRED)
			,'STRIKE': (INLINE_TYPE, TAG_REQUIRED)
			,'STRONG': (INLINE_TYPE, TAG_REQUIRED)
			,'STYLE': (BLOCK_TYPE, TAG_REQUIRED)		# Flagged as block to avoid <br> tags being added.
			,'SUB': (INLINE_TYPE, TAG_REQUIRED)
			,'SUP': (INLINE_TYPE, TAG_REQUIRED)
			,'TABLE': (BLOCK_TYPE, TAG_REQUIRED)
			,'TBODY': (BLOCK_TYPE, TAG_OPTIONAL)
			,'TD': (INLINE_TYPE, TAG_OPTIONAL)
			,'TEXTAREA': (INLINE_TYPE, TAG_REQUIRED)
			,'TFOOT': (INLINE_TYPE, TAG_OPTIONAL)
			,'TH': (INLINE_TYPE, TAG_OPTIONAL)
			,'THEAD': (INLINE_TYPE, TAG_OPTIONAL)
			,'TR': (INLINE_TYPE, TAG_OPTIONAL)
			,'TT': (INLINE_TYPE, TAG_REQUIRED)
			,'U': (INLINE_TYPE, TAG_REQUIRED)
			,'UL': (BLOCK_TYPE, TAG_REQUIRED)
			,'VAR': (INLINE_TYPE, TAG_REQUIRED)
		   }

ESCAPED_TEXT_REGEX=re.compile (r"\&\S+?;")

def writeStartTag (stream, tag, attributes):
	stream.write ('<')
	stream.write (tag)
	for name, value in attributes:		
		stream.write (' ')
		stream.write (name)
		stream.write ('="')
		if (ESCAPED_TEXT_REGEX.search (value) is not None):
			# We already have some escaped characters in here, so assume it's all valid
			stream.write (value)
		else:
			stream.write (cgi.escape (value))
		stream.write ('"')
	stream.write ('>')

class ContentParseException (Exception):
	def __init__ (self, msg):
		self.msg = msg

	def __str__ (self):
		return self.msg

class ContentToHTMLConverter (sgmllib.SGMLParser):
	""" Convert entered markup into HTML.  Paragraph and line break elements are added to the content,
		taking into consideration any block level markup that might have been entered by the user.
	""" 
	def __init__ (self):
		self.log = logging.getLogger ("PubTal.HTMLText.ContentToHTMLConverter")
		sgmllib.SGMLParser.__init__ (self)
		
	def convertContent (self, content):
		self.openBlock = 0
		self.tagStack = []
		self.result = StringIO.StringIO()
		self.currentParagraph = StringIO.StringIO()
		self.feed (content.strip())

		# See if there is anything not yet written out
		# We need to write out the old paragraph first
		oldPara = self.currentParagraph.getvalue()
		if (len (oldPara.strip()) > 0):
			# The old paragraph had content - so write it out
			self.result.write ('<p>')
			# Need to add <br> elements as required
			lines = oldPara.split ('\n')
			for line in lines[0:-1]:
				self.result.write (line)
				if (len (line.strip()) > 0):
						self.result.write ('<br>\n')
			# The last line just gets written out - no further BR
			self.result.write (lines[-1])
			self.result.write ('</p>\n')
				
		return self.result.getvalue()

	def unknown_starttag (self, tag, attributes):
		try:
			tagType, endTagRequired = HTML_TAGS [tag.upper()]
		except KeyError, e:
			raise ContentParseException ("Unhandled tag %s" % tag)

		if (tagType == BLOCK_TYPE):
			# We are starting a block, ensure there are no open inline elements
			# If there are any with optional close tags, close them now.
			looking = len (self.tagStack)
			while (looking):
				oldTag = self.tagStack [-1]
				oldTagType, oldTagRequired = HTML_TAGS [oldTag]
				if (oldTagRequired == TAG_OPTIONAL):
					if (oldTagType == BLOCK_TYPE):
						self.openBlock -= 1
					# Optional, so pop it off the stack and try again
					self.tagStack.pop()
				else:
					# End tag is not optional, so keep going
					looking = 0
				if (len (self.tagStack) == 0):
					looking = 0
	  
		if (endTagRequired != TAG_FORBIDDEN):
			self.tagStack.append (tag.upper())	
	  
		if (self.openBlock > 0):
			# We are writing out to the result directly
			if (tagType == BLOCK_TYPE):
				self.openBlock += 1
			writeStartTag (self.result, tag, attributes)
			return

		# We have a paragraph open
		if (tagType == BLOCK_TYPE):
			self.openBlock += 1
			# We need to write out the old paragraph first
			oldPara = self.currentParagraph.getvalue()
			if (len (oldPara.strip()) > 0):
				# The old paragraph had content - so write it out
				self.result.write ('<p>')
				# Need to add <br> elements as required
				lines = oldPara.split ('\n')
				for line in lines[0:-1]:
					self.result.write (line)
					if (len (line.strip()) > 0):
						self.result.write ('<br>\n')
				# The last line just gets written out - no further BR
				self.result.write (lines[-1])
				self.result.write ('</p>\n')
			self.currentParagraph = StringIO.StringIO()
			# Now in an open block, so write directly to the result stream
			writeStartTag (self.result, tag, attributes)
			return
			
		# Still in a paragraph, so write into it.
		writeStartTag (self.currentParagraph, tag, attributes)

	def unknown_endtag (self, tag):
		looking = 1
		if (self.openBlock > 0):
			self.result.write ('</')
			self.result.write (tag)
			self.result.write ('>')
		else:
			self.currentParagraph.write ('</')
			self.currentParagraph.write (tag)
			self.currentParagraph.write ('>')
		
		while (looking):
			if (len (self.tagStack) > 0):
				oldTag = self.tagStack.pop()
				try:
					tagType, endTagRequired = HTML_TAGS [oldTag.upper()]
				except KeyError, e:
					raise ContentParseException ("Unhandled tag %s" % oldTag)

				if (tagType == BLOCK_TYPE):
					# We've just closed a block element
					self.openBlock -= 1
				if (oldTag == tag.upper()):
					if (endTagRequired == TAG_FORBIDDEN):
						raise ContentParseException ("Tag '%s' can not have an end tag." % tag)
					looking = 0
				else:
					if (endTagRequired == TAG_REQUIRED):
						raise ContentParseException ("Tag '%s' must have an end tag." % oldTag)	 
			else:
				self.log.warn ("Stack empty!")
				# Run out of tags!
				looking = 0
					
	def handle_data (self, data):
		if (self.openBlock > 0):
			# We are in a block, so just output
			self.result.write (cgi.escape (data))
			return
		
		# We are not in a block, so let's do the paragraph thing.
		paraData = data.split ('\n\n')
		for para in paraData [0:-1]:
			# We have a double newline entry
			self.currentParagraph.write (cgi.escape (para))
			
			# We have an open paragraph and got more data, close the old one off
			oldPara = self.currentParagraph.getvalue()
			if (len (oldPara.strip()) > 0):
				# The old paragraph had content - so write it out
				self.result.write ('<p>')
				# Need to add <br> elements as required
				lines = oldPara.split ('\n')
				for line in lines[0:-1]:
					self.result.write (line)
					if (len (line.strip()) > 0):
						self.result.write ('<br>\n')
				# The last line just gets written out - no further BR
				self.result.write (lines[-1])
				self.result.write ('</p>\n')
			self.currentParagraph = StringIO.StringIO()
			
		# The last paragraph doesn't have a newline after it, so just write it out
		self.currentParagraph.write (cgi.escape (paraData[-1]))
		
	def handle_charref (self, ref):
		data = u'&#%s;' % ref
		if (self.openBlock > 0):
			# We are in a block, so just output
			self.result.write (data)
		else:
			# Write to the paragraph
			self.currentParagraph.write (data)
		
	def handle_entityref (self, ref):
		data = u'&%s;' % ref
		if (self.openBlock > 0):
			# We are in a block, so just output
			self.result.write (data)
		else:
			# Write to the paragraph
			self.currentParagraph.write (data)

	def report_unbalanced (self, tag):
			raise ContentParseException ("Recieved close tag '%s', but no corresponding open tag." % tag)

