""" Classes to handle configuration of a PubTal site.

	Copyright (c) 2003 Colin Stewart (http://www.owlfish.com/)
	All rights reserved.
		
	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions
	are met:
	1. Redistributions of source code must retain the above copyright
	   notice, this list of conditions and the following disclaimer.
	2. Redistributions in binary form must reproduce the above copyright
	   notice, this list of conditions and the following disclaimer in the
	   documentation and/or other materials provided with the distribution.
	3. The name of the author may not be used to endorse or promote products
	   derived from this software without specific prior written permission.
	
	THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
	IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
	OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
	IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
	INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
	THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
	
	If you make any bug fixes or feature enhancements please let me know!

"""

try:
	import logging
except:
	from simpletal import DummyLogger as logging
	
import ConfigurationParser
import os, os.path, stat, re
	
class SiteConfig:
	def __init__ (self, configFile):
		self.log = logging.getLogger ("PubTal.SiteConfig")
		self.fileTypeMap = {}
		self.fileMap = {}
		self.directoryMap = {}
		# List of regular expressions that match files to be ignored
		self.ignoreFilters = []
		self.baseDir = os.path.abspath (os.path.split (configFile)[0])
		
		# Defaults
		self.setCharacterSet ('ISO8859-15')
		self.contentDir = os.path.join (self.baseDir, 'content')
		self.destDir = os.path.join (self.baseDir, 'dest')
		self.templateDir = os.path.join (self.baseDir, 'template')
		
		txtConfig = PageConfigItem()
		txtConfig.setOption ('content-type', 'HTMLText')
		self.fileTypeMap ['txt'] = txtConfig
		
		catalogueConfig = PageConfigItem()
		catalogueConfig.setOption ('content-type', 'Catalogue')
		self.fileTypeMap ['catalogue'] = catalogueConfig
		
		self.readConfig(configFile)
		
	def getContentDir (self):
		return self.contentDir
		
	def getDestinationDir (self):
		return self.destDir
		
	def getTemplateDir (self):
		return self.templateDir
		
	def getIgnoreFilters (self):
		return self.ignoreFilters
		
	def getCharacterSet (self):
		return self.characterSet
		
	def setCharacterSet (self, charSet):
		self.characterSet = charSet
		
	def getPage (self, contentPath):
		page = Page (contentPath, self.contentDir, self.destDir, self.templateDir)
		# Look for file type first
		configItem = self.fileTypeMap.get (contentPath [contentPath.rfind ('.')+1:])
		if (configItem is not None):
			configItem.updatePage (page)
		
		# Now work through directory map
		dirConfigItems = []
		head, tail = os.path.split (contentPath)
		while (tail != ''):
			self.log.debug ("Looking for directory configuration at: %s" % head)
			configItem = self.directoryMap.get (head, None)
			if (configItem is not None):
				dirConfigItems.insert (0,configItem)
			if (self.contentDir.startswith (head)):
				# We have reached the top of the content dir, so stop now
				tail = ''
			else:
				head, tail = os.path.split (head)
			
		for confItem in dirConfigItems:
			self.log.debug ("Updating config using %s" % str (confItem))
			confItem.updatePage (page)
		
		# Now look for this specific file
		configItem = self.fileMap.get (contentPath)
		if (configItem is not None):
			configItem.updatePage (page)
			
		return page
		
	def readConfig (self, configFile):
		parser = ConfigurationParser.ConfigurationParser()
		parser.addHandler ('SiteConfig', self)
		parser.addHandler ('FileType', self)
		parser.addHandler ('Directory', self)
		parser.addHandler ('File', self)
		self.currentDirective = None
		self.currentConfigItem = None
		confFile = open (configFile, 'r')
		parser.parse(confFile)
		confFile.close()
		
		for fileType in self.fileTypeMap.keys():
			self.log.debug ("FileType %s has Configuration: %s" % (fileType, str (self.fileTypeMap [fileType])))
		for dir in self.directoryMap.keys():
			self.log.debug ("Directory %s has Configuration: %s" % (dir, str (self.directoryMap [dir])))
		for file in self.fileMap.keys():
			self.log.debug ("File %s has Configuration: %s" % (file, str (self.fileMap [file])))
		
	def startDirective (self, directive, options):
		self.currentDirective = directive
		if (directive == 'FILETYPE'):
			self.currentConfigItem = PageConfigItem()
			self.fileTypeMap [options] = self.currentConfigItem
		elif (directive == 'DIRECTORY'):
			self.currentConfigItem = PageConfigItem()
			targetDir = os.path.join (self.contentDir, options)
			if (targetDir.endswith (os.sep)):
				targetDir = targetDir [:-1]
			self.directoryMap [targetDir] = self.currentConfigItem
		elif (directive == 'FILE'):
			self.currentConfigItem = PageConfigItem()
			self.fileMap [os.path.join (self.contentDir, options)] = self.currentConfigItem
		else:
			self.currentConfigItem = None
		
	def endDirective (self, directive):
		self.currentConfigItem = None
		self.currentDirective = None
		
	def option (self, line):
		if (self.currentConfigItem is not None):
			if (line.lower().startswith ('macro')):
				mnStart = line.find (' ')
				mnEnd = line.find (' ', mnStart+1)
				macroName = line [mnStart+1:mnEnd]
				macro = line [mnEnd + 1:]
				self.currentConfigItem.addMacro (macroName, os.path.join (self.templateDir, macro))
			elif (line.lower().startswith ('header')):
				mnStart = line.find (' ')
				mnEnd = line.find (' ', mnStart+1)
				headerName = line [mnStart+1:mnEnd]
				header = line [mnEnd + 1:]
				self.currentConfigItem.addHeader (headerName, header)
			else:
				firstSpace = line.find (' ')
				name = line [:firstSpace]
				value = line [firstSpace+1:]
				self.currentConfigItem.setOption (name, value)
				
		elif (self.currentDirective == 'SITECONFIG'):
			if (line.lower().startswith ('content-dir')):
				self.contentDir = os.path.join (self.baseDir, line [line.find (' ')+1:])
			elif (line.lower().startswith ('template-dir')):
				self.templateDir = os.path.join (self.baseDir, line [line.find (' ')+1:])
			elif (line.lower().startswith ('dest-dir')):
				self.destDir = os.path.join (self.baseDir, line [line.find (' ')+1:])
			elif (line.lower().startswith ('ignore-filter')):
				filter = line [line.find (' ')+1:]
				self.log.info ("Adding filter of content to ignore: %s" % filter)
				self.ignoreFilters.append (re.compile (filter))
			elif (line.lower().startswith ('character-set')):
				self.setCharacterSet (line[line.find (' ') + 1:])
			else:
				self.log.warn ("SiteConfig Option %s not supported" % line)
		
class PageConfigItem:
	def __init__ (self):
		self.macros = {}
		self.headers = {}
		self.options = {}
		self.template = None
		self.contentType = None
		
	def updatePage (self, page):
		page.macros.update (self.macros)
		page.headers.update (self.headers)
		page.options.update (self.options)
			
	def addMacro (self, name, template):
		self.macros [name] = template
		
	def addHeader (self, name, header):
		self.headers [name] = header
		
	def setOption (self, name, value):
		self.options [name] = value
		
	def __str__ (self):
		desc = ""
		if (len (self.macros) > 0):
			desc += "Macros: %s\n" % str (self.macros)
		if (len (self.headers) > 0):
			desc += "Headers: %s\n" % str (self.headers)
		if (self.contentType is not None):
			desc += "Content Type: %s" % self.contentType
		return desc

class Page:
	def __init__ (self, source, contentDir, destDir, templateDir):
		# Things we need
		self.source = source
		self.macros = {}
		self.options = {'template': 'template.html'}
		self.headers = {}
		self.destDir = destDir
		
		commonRoot = os.path.commonprefix ([contentDir, source])
		self.relativePath = source[len (commonRoot)+1:]
		# relativePath is of the form: [dir/]file
		depth = 0
		head, tail = os.path.split (self.relativePath)
		while (tail != ''):
			depth += 1
			head, tail = os.path.split (head)
		# We go to far, so knock one off!
		self.depth = depth - 1
		
	def getOption (self, name, defaultValue=None):
		return self.options.get (name, defaultValue)

	def getSource (self):
		return self.source
		
	def getRelativePath (self):
		return self.relativePath
		
	def getMarcos (self):
		return self.macros
		
	def getHeaders (self):
		return self.headers
		
	def getDepthString (self):
		return "../"*self.depth
		
	def getModificationTime (self):
		""" Returns a tuple of the creation and modification date/time. """
		info = os.stat (self.source)
		return info[stat.ST_MTIME]
		
	def __str__ (self):
		return self.source
		
