""" Classes to handle publishing of a PubTal site.

	Copyright (c) 2003 Colin Stewart (http://www.owlfish.com/)
	All rights reserved.
		
	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions
	are met:
	1. Redistributions of source code must retain the above copyright
	   notice, this list of conditions and the following disclaimer.
	2. Redistributions in binary form must reproduce the above copyright
	   notice, this list of conditions and the following disclaimer in the
	   documentation and/or other materials provided with the distribution.
	3. The name of the author may not be used to endorse or promote products
	   derived from this software without specific prior written permission.
	
	THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
	IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
	OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
	IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
	INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
	THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
	
	If you make any bug fixes or feature enhancements please let me know!

"""

try:
	import logging
except:
	from simpletal import DummyLogger as logging
	
import time, codecs, os, os.path

from simpletal import simpleTAL, simpleTALES, simpleTALUtils
import pubtal

import ContentToHTMLConverter, ArticleContent, CatalogueContent

class PagePublisher:
	def __init__ (self, config):
		self.templateCache = simpleTALUtils.TemplateCache()
		self.config = config
		self.htmlTextPublisher = HTMLTextPagePublisher (self)
		self.cataloguePublisher = CataloguePublisher (self)
		self.log = logging.getLogger ("PubTal.PagePublisher")
		
	def getTemplateCache (self):
		return self.templateCache, self.config.getCharacterSet()
		
	def getConfig (self):
		return self.config
		
	def publish (self, page):
		contentType = page.getOption ('content-type')
		if (contentType == "HTMLText"):
			self.htmlTextPublisher.publish (page)
		elif (contentType == "Catalogue"):
			self.cataloguePublisher.publish (page)
		else:
			self.log.warn ("Unsupported content type.")
		
class ContextFunctions:
	def __init__ (self):
		self.currentPage = None
		self.isPage = simpleTALES.PathFunctionVariable (self.isCurrentPage)
		
	def setCurrentPage (self, page):
		self.currentPage = page
		
	def isCurrentPage (self, otherPage):
		if (self.currentPage == otherPage):
			return 1
		return 0

class ContentPublisher:
	def __init__ (self, pagePublisher):
		self.pagePublisher = pagePublisher
		self.templateCache, self.characterSet = pagePublisher.getTemplateCache()
		self.characterSetCodec = codecs.lookup (self.characterSet)[1]
		self.destDir = pagePublisher.getConfig().getDestinationDir()
		self.templateDir = pagePublisher.getConfig().getTemplateDir()
		self.converter = ContentToHTMLConverter.ContentToHTMLConverter()
				
		# Used in the Context
		self.ContextFunctions = ContextFunctions()
		self.pubTalInfo = {'version': pubtal.__version__
							,'url': "http://www.owlfish.com/software/PubTal/"
							,'linkText': """<p title="PubTal is a template driven web site publisher.">Made with <a href="http://www.owlfish.com/software/PubTal/">PubTal</a> %s</p>""" % pubtal.__version__
						}
						
class CataloguePublisher (ContentPublisher):
	def __init__ (self, pagePublisher):
		ContentPublisher.__init__ (self, pagePublisher)
		self.log = logging.getLogger ("PubTal.CataloguePublisher")
		
	def publish (self, page):
		indexTemplateName = os.path.join (self.templateDir, page.getOption ('catalogue-index-template', 'template.html'))
		itemTemplateName = os.path.join (self.templateDir, page.getOption ('catalogue-item-template', 'template.html'))
		maxCols = int (page.getOption ('catalogue-max-columns', '5'))
		buildIndexPage = 0
		buildItemPages = 0
		catalogueBuildPages = page.getOption ('catalogue-build-pages', 'index,item')
		for option in catalogueBuildPages.split (','):
			if (option == "index"):
				if (indexTemplateName is not None):
					buildIndexPage = 1
				else:
					self.log.warn ("Unable to build the index page for catalogue %s because no catalogue-index-template has been specified." % page.getSource())
			elif (option == "item"):
				if (itemTemplateName is not None):
					buildItemPages = 1
				else:
					self.log.warn ("Unable to build the item pages for catalogue %s because no catalogue-item-template has been specified." % page.getSource())
		
		if (not buildIndexPage | buildItemPages):
			self.log.warn ("Neither index or item pages are being built for catalogue %s" % page.getSource())
			return
			
		# Build the context pieces we are going to need
		catalogue = CatalogueContent.CatalogueContent (page.getSource(), self.characterSetCodec)
		items = []
		rows = []
		col = []
		lastModDate = time.strftime ('%a, %d %b %Y %H:%M:%S %Z', time.localtime (page.getModificationTime()))
		copyrightYear = time.strftime ('%Y')
		# Source paths
		relativeSourcePath = page.getRelativePath()
		absSourcePath = page.getSource()
		localDestinationDir = os.path.dirname (page.getRelativePath())
		depth = page.getDepthString()
		for itemHeaders in catalogue.getItems():
			# Destination paths
			filename = itemHeaders.get ('filename', None)
			if (filename is None):
				self.log.error ("Unable to publish catalogue %s.  Missing filename header in catalogue." % page.getSource())
				return
			
			relativeDestPath = os.path.join (localDestinationDir, os.path.splitext (filename)[0] + '.html')
			destPath = os.path.join (self.destDir, relativeDestPath)
			destFilename = os.path.basename (destPath)
			actualHeaders = {}
			actualHeaders.update (page.getHeaders())
			actualHeaders.update (itemHeaders)
			itemmap = {	'lastModifiedDate': lastModDate
				,'copyrightYear': copyrightYear
				,'sourcePath': relativeSourcePath
				,'absoluteSourcePath': absSourcePath
				,'destinationPath': relativeDestPath
				,'absoluteDestinationPath': destPath
				,'destinationFilename': destFilename
				,'depth': depth
				,'headers': actualHeaders
				}
			items.append (itemmap)
			if (len (col) == maxCols):
				rows.append (col)
				col = []
			col.append (itemmap)
		if (len (col) > 0):
			rows.append (col)
			col = []
		
		# Build the Catalogue context
		catalogueMap = {'items': items, 'rows': rows, 'headers': catalogue.getCatalogueHeaders()}
		# Do the individual items now
		if (buildItemPages):
			template = self.templateCache.getTemplate (itemTemplateName, inputEncoding=self.characterSet)
			itemCount = 0
			itemLength = len (items)
			for item in items:
				relativeDestPath = item['destinationPath']
				destPath = item['absoluteDestinationPath']
				context = simpleTALES.Context(allowPythonPath=1)
				self.ContextFunctions.setCurrentPage (relativeDestPath)
				context.addGlobal ('ispage', self.ContextFunctions.isPage)
				context.addGlobal ('pubtal', self.pubTalInfo)
				context.addGlobal ('page', item)
				if (itemCount > 0):
					catalogueMap ['previous'] = items[itemCount - 1]
				elif (catalogueMap.has_key ('previous')):
					del catalogueMap ['previous']
				if (itemCount < itemLength - 1):
					catalogueMap ['next'] = items[itemCount + 1]
				elif (catalogueMap.has_key ('next')):
					del catalogueMap ['next']
					
				context.addGlobal ('catalogue', catalogueMap)
				
				# Add macros to the context
				macroTemplates = {}
				macros = page.getMarcos()
				for macroName in macros.keys():
					mTemp = self.templateCache.getTemplate (macros [macroName], inputEncoding=self.characterSet)
					macroTemplates [macroName] = mTemp.macros
				
				context.addGlobal ('macros', macroTemplates)
				
				# Make directories if required.
				destDir = os.path.split (destPath)[0]
				if (not os.path.exists (destDir)):
					os.makedirs (destDir)
				dest = open (destPath, 'w')
				template.expand (context, dest)
				dest.close()
				itemCount += 1
				
		if (buildIndexPage):
			# Cleanup the catalogueMap from the items pages.
			if (catalogueMap.has_key ('previous')):
				del catalogueMap ['previous']
			if (catalogueMap.has_key ('next')):
				del catalogueMap ['next']
			template = self.templateCache.getTemplate (indexTemplateName, inputEncoding=self.characterSet)
			
			relativeDestPath = os.path.splitext (page.getRelativePath())[0] + '.html'
			destPath = os.path.join (self.destDir, relativeDestPath)
			destFilename = os.path.basename (destPath)
			actualHeaders = page.getHeaders()
			actualHeaders.update (catalogue.getCatalogueHeaders())
			indexMap = {'lastModifiedDate': lastModDate
						,'copyrightYear': copyrightYear
						,'sourcePath': relativeSourcePath
						,'absoluteSourcePath': absSourcePath
						,'destinationPath': relativeDestPath
						,'absoluteDestinationPath': destPath
						,'destinationFilename': destFilename
						,'depth': depth
						,'headers': actualHeaders
						}
			context = simpleTALES.Context(allowPythonPath=1)
			self.ContextFunctions.setCurrentPage (relativeDestPath)
			context.addGlobal ('ispage', self.ContextFunctions.isPage)
			context.addGlobal ('pubtal', self.pubTalInfo)
			context.addGlobal ('page', indexMap)
				
			context.addGlobal ('catalogue', catalogueMap)
			
			# Add macros to the context
			macroTemplates = {}
			macros = page.getMarcos()
			for macroName in macros.keys():
				mTemp = self.templateCache.getTemplate (macros [macroName], inputEncoding=self.characterSet)
				macroTemplates [macroName] = mTemp.macros
			
			context.addGlobal ('macros', macroTemplates)
			
			# Make directories if required.
			destDir = os.path.split (destPath)[0]
			if (not os.path.exists (destDir)):
				os.makedirs (destDir)
			dest = open (destPath, 'w')
			template.expand (context, dest)
			dest.close()
		
		
class HTMLTextPagePublisher (ContentPublisher):
	def __init__ (self, pagePublisher):
		ContentPublisher.__init__ (self, pagePublisher)
		self.log = logging.getLogger ("PubTal.HTMLTextPagePublisher")
		
	def publish (self, page):
		templateName = os.path.join (self.templateDir, page.getOption ('template'))
		template = self.templateCache.getTemplate (templateName, inputEncoding=self.characterSet)
		context = simpleTALES.Context(allowPythonPath=1)
		headers, rawContent = self.getRawContent(page)
		content = self.converter.convertContent (rawContent)
		article = ArticleContent.ArticleContent ()
		article.setArticleBody (content)
		modTime = time.localtime (page.getModificationTime())
		article.modificationDate = modTime
		article.creationDate = modTime
		
		# Determine the destination for this page
		relativeDestPath = os.path.splitext (page.getRelativePath())[0] + '.html'
		destPath = os.path.join (self.destDir, relativeDestPath)
		destFilename = os.path.basename (destPath)
		
		self.ContextFunctions.setCurrentPage (relativeDestPath)
		context.addGlobal ('ispage', self.ContextFunctions.isPage)
		context.addGlobal ('pubtal', self.pubTalInfo)
		
		map = {	'lastModifiedDate': article.getRFC822Date()
				,'copyrightYear': article.getCopyrightYear()
				,'content': content
				,'rawContent': rawContent
				,'sourcePath': page.getRelativePath()
				,'absoluteSourcePath': page.getSource()
				,'destinationPath': relativeDestPath
				,'absoluteDestinationPath': destPath
				,'destinationFilename': destFilename
				,'depth': page.getDepthString()
				}
		
		# Merge in configuration headers and per-page headers
		actualHeaders = page.getHeaders()
		actualHeaders.update (headers)
		map ['headers'] = actualHeaders
		self.log.debug (str (headers))
		context.addGlobal ('page', map)
		
		# Add macros to the context
		macroTemplates = {}
		macros = page.getMarcos()
		for macroName in macros.keys():
			mTemp = self.templateCache.getTemplate (macros [macroName], inputEncoding=self.characterSet)
			macroTemplates [macroName] = mTemp.macros
		
		context.addGlobal ('macros', macroTemplates)
		
		
		# Make directories if required.
		destDir = os.path.split (destPath)[0]
		if (not os.path.exists (destDir)):
			os.makedirs (destDir)
		dest = open (destPath, 'w')
		template.expand (context, dest)
		dest.close()
	
	def getRawContent (self, page):
		sourceFile = open (page.getSource(), 'r')
		readingHeaders = 1
		headers = {}
		while (readingHeaders):
			line = self.characterSetCodec (sourceFile.readline())[0]
			offSet = line.find (':')
			if (offSet > 0):
				headers [line[0:offSet]] = line[offSet + 1:].strip()
			else:
				readingHeaders = 0
		rawContent = self.characterSetCodec (sourceFile.read())[0]
		sourceFile.close()
		return (headers, rawContent)