""" Utility classes to help automate the PubTal testing.

	Copyright (c) 2003 Colin Stewart (http://www.owlfish.com/)
	All rights reserved.
		
	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions
	are met:
	1. Redistributions of source code must retain the above copyright
	   notice, this list of conditions and the following disclaimer.
	2. Redistributions in binary form must reproduce the above copyright
	   notice, this list of conditions and the following disclaimer in the
	   documentation and/or other materials provided with the distribution.
	3. The name of the author may not be used to endorse or promote products
	   derived from this software without specific prior written permission.
	
	THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
	IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
	OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
	IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
	INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
	THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
	
	If you make any bug fixes or feature enhancements please let me know!

"""

try:
	import logging
except:
	from simpletal import DummyLogger as logging
	
import os, os.path, copy

class SiteBuilder:
	def __init__ (self, location=None):
		self.log = logging.getLogger ("PubTal.SiteCreation")
		if (location is None):
			self.siteDir = os.tempnam()
		else:
			self.siteDir = location
		
		if (os.access (self.siteDir, os.F_OK)):
			msg = "Directory %s already exists!" % self.siteDir
			self.log.error (msg)
			raise Exception (msg)
	
	def buildDirs (self, templateDir="template", destinationDir="dest", contentDir="content"):
		self.log.debug ("Building site directory %s" % self.siteDir)
		os.mkdir (self.siteDir)
		
		self.contentDir = os.path.join (self.siteDir, contentDir)
		self.log.debug ("Building content directory %s" % self.contentDir)
		os.mkdir (self.contentDir)
		
		self.destinationDir = os.path.join (self.siteDir, destinationDir)
		self.log.debug ("Building destination directory %s" % self.destinationDir)
		os.mkdir (self.destinationDir)
			
		self.templateDir = os.path.join (self.siteDir, templateDir)
		self.log.debug ("Building template directory %s" % self.templateDir)
		os.mkdir (self.templateDir)
		
	def createContent (self, filePath, content):
		self.log.debug ("Creating content file %s" % filePath)		
		destPath = os.path.join (self.contentDir, filePath)
		self._createDirsAndFile_ (destPath, content)
		
	def createTemplate (self, filePath, template):
		self.log.debug ("Creating template file %s" % filePath)		
		destPath = os.path.join (self.templateDir, filePath)
		self._createDirsAndFile_ (destPath, template)
		
	def createConfigFile (self, filePath, config):
		self.log.debug ("Creating configuration file %s" % filePath)		
		destPath = os.path.join (self.siteDir, filePath)
		self._createDirsAndFile_ (destPath, config)
		
	def getSiteDir (self):
		return self.siteDir
		
	def getContentDir (self):
		return self.contentDir
		
	def getDestDir (self):
		return self.destinationDir
		
	def _createDirsAndFile_ (self, destPath, content):
		# Make directories if required.
		destDir = os.path.split (destPath)[0]
		if (not os.path.exists (destDir)):
			os.makedirs (destDir)
		dest = open (destPath, 'w')
		dest.write (content)
		dest.close()
		
	def destroySite (self):
		self.log.debug ("Destroying site directory and contents")
		pathCleaner = pathRemover ()
		pathCleaner.walk (self.siteDir)
		
class pathRemover:
	def __init__ (self):
		self.dirsToRemove = []
		self.log = logging.getLogger ("PubTal.SiteCreation.pathRemover")
		
	def walk (self, path):
		self.dirsToRemove = [path]
		os.path.walk (path, self.walking, None)
		# Now remove all of the directories we saw, starting with the last one
		self.dirsToRemove.reverse()
		for dir in self.dirsToRemove:
			os.rmdir (dir)
		self.dirsToRemove = []
		
	def walking (self, arg, dirname, names):
		for name in names:
			#self.log.debug ("Would delete file: %s" % os.path.join (dirname, name))
			target = os.path.join (dirname, name)
			if (os.path.islink (target)):
				os.remove (target)
			elif (os.path.isfile (target)):
				os.remove (target)
			elif (os.path.isdir (target)):
				self.dirsToRemove.append (target)
			else:
				self.log.error ("Path %s is neither a directory or a file!" % target)

class DirCompare:
	def compare (self, path, expected):
		self.expected = copy.copy (expected)
		self.path = path
		self.badFile = None
		
		os.path.walk (path, self.walking, None)
		
		if (self.badFile is not None):
			return self.badFile
			
		if (len (self.expected) > 0):
			return "Missing files: " + str (self.expected.keys())
		return None
		
	def walking (self, arg, dirname, names):
		if (self.badFile is not None):
			return
		commonRoot = os.path.commonprefix ([self.path, dirname])
		for name in names:
			target = os.path.join (dirname, name)
			relTarget = target[len (commonRoot)+1:]
			if (os.path.isfile (target)):
				if (not self.expected.has_key (relTarget)):
					self.badFile = "Found unexepected file %s" % relTarget
					return
					
				testFile = open (target, 'r')
				content = testFile.read()
				testFile.close()
				if (content != self.expected [relTarget]):
					self.badFile = "File %s had content:\n%s\nexpected:\n%s\n" % (relTarget, content, self.expected [relTarget])
					return
				del self.expected [relTarget]
					
